#!/bin/bash

#updates by WIBOON 2558-01-29, 2561-07-10

#Add multiple users with bulk import
#You can import multiple users into the file server from a single file source. The file must be a text file and adhere to the required format. 
#For example 
#Filename: usergroup1
#user1
#user2
#user3
#
#When finished, the original filename will be replace with a filename_timestamp.csv consists of lines with username:password format.

exitusage()
{
  echo "usage: sudo sh $0 path/to/filename"
  echo "path/to/filename is a file consists of one username per line i.e."
  echo "user1"
  echo "user2"
  exit 
}

if [ -z "$(which apg)" ] ; then
  echo "need program apg, try: sudo apt-get install apg"
  exit
fi
if [ -z "$(which mkpasswd)" ] ; then
  echo "need program mkpasswd, try: sudo apt-get install whois"
  exit
fi
if [ -z "$(id -nG|grep -w "root")" ] ; then
  echo "need root, or try: sudo sh $0"
  exit
fi
if [ "$1" = "--help" ] ; then
  exitusage
fi
if [ -z "$1" ] ; then
  echo "need filename"
  exitusage
fi

TODAY=$(date "+%Y%m%d%H%M")
IN_FILENAME=$1
HOME_DIR="/home"
OUT_FILENAME=${IN_FILENAME}_${TODAY}.csv

if [ ! -f ${IN_FILENAME} ] ; then
  echo "File not found"
  exit
fi

sudo touch ${OUT_FILENAME}

for USERDETAILS in $(cat ${IN_FILENAME})
do
  MYUSER="$(echo ${USERDETAILS})"
  PASSWD="$(apg -n 1 -a 1 -M nL -m 10 -x 10 -E ijl0oqg)"
  echo "...Processing adding user ${MYUSER}"
  if [ ! -z "$(getent passwd ${MYUSER})" ]; then
    echo "username exist"
    sudo echo "${MYUSER}:invalid" >> ${OUT_FILENAME}
  else
    sudo useradd -m -b ${HOME_DIR} -s /bin/bash ${MYUSER} -p `mkpasswd ${PASSWD}`
    sudo echo "${MYUSER}:${PASSWD}" >> ${OUT_FILENAME}
  fi
  sleep 1
done

isnologin=false
if [ -f /sbin/nologin ] ; then
  isnologin=true
  NOLOGIN=/sbin/nologin
else 
  if [ -f /usr/sbin/nologin ] ; then
    isnologin=true
    NOLOGIN=/usr/sbin/nologin
  else 
    if [ -f /bin/false ] ; then
      isnologin=true
      NOLOGIN=/bin/false
    fi
  fi
fi
if [ "${isnologin}" = "true" ]; then
  while true; do
    echo " "
    echo "Do you want these users can remote access with ssh to server?"
    echo "Answer y = have username and password and can remote access with ssh to server"
    echo -n "Answer n = have username and password only"
    read -p " {y/n)" yn
    case $yn in
        [Yy]* ) CONFIRM_ANS="yes"; break;;
        [Nn]* ) CONFIRM_ANS="no"; break;;
        * ) echo "Please answer y or n.";;
    esac
  done
  if [ "${CONFIRM_ANS}" = "no" ]; then
    echo "Modify user shell from /bin/bash to ${NOLOGIN}"
    for USERDETAILS in $(cat ${OUT_FILENAME})
    do
      MYUSER="$(echo ${USERDETAILS}|cut -d':' -f1)"
      if [ ! -z "$(getent passwd ${MYUSER})" ]; then
        echo "modifying username ${MYUSER}"
        sudo usermod -s ${NOLOGIN} ${MYUSER}
      fi
      sleep 1
    done
  fi
fi

sudo chmod go-rwx ${OUT_FILENAME}
sudo rm -f ${IN_FILENAME}
echo " "
echo "Finished, the original filename ${IN_FILENAME} replaced with ${OUT_FILENAME} consists of lines with username:password format."
#END

